# vim: set fileencoding=utf-8 :
from django.contrib import admin
from logistics.models import *


class ServiceObjectInlineAdmin(admin.TabularInline):
	model = ServiceObject
	extra = 15

class EngineerAdmin(admin.ModelAdmin):
	fieldsets = (
		(u'Основная информация', {'fields': (
						'login',
						'password',
						'lastname',
						'firstname',
						'middlename',
						'photo',
						'work_tel',
						'mobile_tel',
						'email',
						'jabber')}),
		(u'Клиенты', 		{'fields': (
						'clients',)}),
		(u'Счет', 		{'fields': (
						'balance',
						'office_base_payment',
						'office_percent_payment',
						'remote_base_payment',
						'remote_percent_payment')}),
		(u'Доп. информация', {'fields': (
						'status',
						'location',
						'target',
						)}),
	)

	list_display = ('lastname', 'firstname', 'middlename', 'balance', 'status', 'target', 'location')
	ordering = ('lastname',)
	search_fields = ('lastname',)
	list_filter = ('clients', 'status')
	def save_model(self, request, obj, form, change):
		obj.save(admin=request.user)
		form.save()

class DispatcherAdmin(admin.ModelAdmin):
	fieldsets = (
		(u'Основная информация', {'fields': (
						'login',
						'password',
						'lastname',
						'firstname',
						'middlename',
						'photo',
						'work_tel',
						'mobile_tel',
						'email',
						'jabber')}),
		(u'Счет', 		{'fields': (
						'balance',
						'base_payment',
						'percent_payment')}),
		(u'Доп. информация', {'fields': (
						'status',
						)}),
	)


	list_display =  ('lastname', 'firstname', 'middlename', 'balance', 'status')
	ordering = ('lastname',)
	search_fields = ('lastname',)
	list_filter = ('status',)
	def save_model(self, request, obj, form, change):
		obj.save(admin=request.user)
		form.save()
	

class TaskAdmin(admin.ModelAdmin):
	list_display = ('contract', 'engineer', 'type', 'description', 'status', 'confirmed', 'get_cost')
	ordering = ('contract',)
	search_fields = ('description',)
	list_filter = ('contract', 'type', 'status')
	def save_model(self, request, obj, form, change):
		obj.save(admin=request.user)
		form.save()

### History admin models
class TaskHistoryAdmin(admin.ModelAdmin):
	list_display = ('date_time', 'task', 'status', 'confirmed', 'admin')
	ordering = ('date_time',)
	list_filter = ('date_time', 'admin')

class EngineerHistoryAdmin(admin.ModelAdmin):
	pass
class DispatcherHistoryAdmin(admin.ModelAdmin):
	pass
### END History admin models

class ServiceLogisticsUparamAdmin(admin.ModelAdmin):
	list_display = ('number', 'type', 'start_date_time', 'end_date_time', 'unlimit_service', 'get_cost')
	list_filter = ('unlimit_service',)
	inlines = (ServiceObjectInlineAdmin,)


admin.site.register(Engineer, EngineerAdmin)
admin.site.register(Dispatcher, DispatcherAdmin)
admin.site.register(Task, TaskAdmin)

admin.site.register(TaskHistory, TaskHistoryAdmin)
admin.site.register(EngineerHistory, EngineerHistoryAdmin)
admin.site.register(DispatcherHistory, DispatcherHistoryAdmin)

admin.site.register(ServiceLogisticsUparam, ServiceLogisticsUparamAdmin)
