# vim: set fileencoding=utf-8 :
from django.db import models
from katrin.models import *
from django.contrib.auth.models import User
from datetime import datetime

import decimal


ENGINEER_STATUS = (
	(0, u'Не доступен'),
	(1, u'Свободен'),
	(2, u'В дороге'),
	(3, u'Работаю'),
)

DISPATCHER_STATUS = (
	(0, u'Не доступен'),
	(1, u'Свободен'),
	(2, u'Работаю'),
)

TASK_STATUS = (
	(0, u'Не обработана'),
	(1, u'Назначена'),
	(2, u'Завершено'),
	(3, u'Успех'),
	(4, u'Провал'),
)

CONTRACT_TYPE = (
	(0, 'Разовое выполнение работ'),
	(1, 'Абоненское обслуживание'),
)

TASK_TYPE = (
	(0, u'Плановая'),
	(1, u'Внеплановая'),
)

TASK_TYPE_PLACE = (
	(0, u'Офис'),
	(1, u'С выездом к клиенту'),
)

#Abstract model
class Employee(Human):
	""" Employee gets salary
	"""
	balance = models.DecimalField(u'Баланс', max_digits=10, decimal_places=4, default=Decimal('0'))
	class Meta:
		abstract = True

class Engineer(Employee):
	clients = models.ManyToManyField(Client, verbose_name=u'Клиент', help_text=u'Инженер закреплен за этими клиентами', null=True, blank=True)
	status = models.IntegerField(u'Статус', choices=ENGINEER_STATUS, default=0)
	office_base_payment = models.DecimalField(u'Офисная базовая оплата', max_digits=10, decimal_places=2, default=0.0)
	office_percent_payment = models.IntegerField(u'Офисный процент', default=0)
	remote_base_payment = models.DecimalField(u'Выездная базовая оплата', max_digits=10, decimal_places=2, default=0.0)
	remote_percent_payment = models.IntegerField(u'Выездной процент', default=0)
	location = models.CharField(u'Местоположение', max_length=200, null=True, blank=True)
	target = models.ForeignKey(Client, verbose_name=u'Цель', null=True, blank=True, related_name='target_set1')
	class Meta:
		verbose_name = u'инженер'
		verbose_name_plural = u'Инженеры'
	def __unicode__(self):
		return "%s %s %s" % (self.lastname, self.firstname, self.middlename)
	def save(self, admin=None):
		eh = EngineerHistory(
				date_time = datetime.now(),
				admin = admin,

				balance = self.balance,
				status = self.status,
				location = self.location,
				target = self.target,
			)
		super(Engineer, self).save()
		eh.engineer = self
		eh.save()

		self.user.groups.add(Group.objects.get(name='logistics_engineers'))
		self.user.save()


class Dispatcher(Employee):
	status = models.IntegerField(u'Статус', choices=DISPATCHER_STATUS, default=0)
	base_payment = models.DecimalField(u'Базовая оплата', max_digits=10, decimal_places=2, default=0.0)
	percent_payment = models.IntegerField(u'Процент', default=0)
	class Meta:
		verbose_name = u'диспетчер'
		verbose_name_plural = u'Диспетчера'
	def save(self, admin=None):
		dh = DispatcherHistory(
				date_time = datetime.now(),
				admin = admin,

				balance = self.balance,
				status = self.status,
			)
		super(Dispatcher, self).save()
		dh.dispatcher = self
		dh.save()

		self.user.groups.add(Group.objects.get(name='logistics_dispatchers'))
		self.user.save()

from katrin import service
#class ServiceLogisticsFilter(service.ServiceFilter):
	#day_hours = models.CharField(max_length=150, null=True, blank=True, help_text='Часы дня в которые применятся фильтр через зяпятую (0-23)')
	#week_days = models.CharField(max_length=60, null=True, blank=True, help_text='Дни недели в которые применятся фильтр через зяпятую (1-7)')
	#month_days = models.CharField(max_length=180, null=True, blank=True, help_text='Дни месяца в которые применятся фильтр через зяпятую (1-31)')
	#year_months = models.CharField(max_length=120, null=True, blank=True, help_text='Месяцы года в которые применятся фильтр через зяпятую (1-12)')
	#additional_coefficient = models.DecimalField(u'Доп. коэффициент', max_digits=10, decimal_places=2, default=1.0)
	#class Meta:
		#db_table = u'service_logistics_filters'
		#verbose_name = 'фильтр трафика'
		#verbose_name_plural = 'Фильтры трафика'

#Contracts
class ServiceLogisticsUparam(service.ServiceUparam):
	number = models.IntegerField(u'Номер контракта', unique=True)
	type = models.IntegerField(u'Тип контракта', choices=CONTRACT_TYPE, default=0)
	start_date_time = models.DateTimeField(u'Начало контракта')
	end_date_time = models.DateTimeField(u'Окончание контракта')
	time_reacting = models.IntegerField(u'Время реакции', default=24, help_text=u'Время реагирования в часах')

	unlimit_service = models.NullBooleanField(u'Неограниченное количество вызовов', null=True, blank=True, help_text=u'При указании отменяет действие следующих четырех параметров')
	planned_service_count = models.IntegerField(u'Количество плановых вызовов', null=True, blank=True)
	exrta_service_count = models.IntegerField(u'Количество ВНЕплановых вызовов', null=True, blank=True)
	planned_service_count_remnant = models.IntegerField(u'Остаток плановых вызовов', null=True, blank=True)
	exrta_service_count_remnant = models.IntegerField(u'Остаток ВНЕплановых вызовов', null=True, blank=True)

	fixed_payment = models.DecimalField(u'Фиксированный платеж', max_digits=10, decimal_places=2, null=True, blank=True, help_text=u'При указании отменяет расчет по полю "Коэффициент" и стоимости обслуживания по объектам контракта')
	coefficient = models.DecimalField(u'Коэффициент', max_digits=7, decimal_places=2, default=1.0)

	last_write_off = models.DateTimeField(u'Время последнего списания', null=True, blank=True)
	class Meta:
		db_table = u'service_logistics_uparams'
		verbose_name = u'контракт'
		verbose_name_plural = u'Контракты'
	def __unicode__(self):
		return u"%s (%s)" % (self.client, self.number)
	def save(self):
		if not self.last_write_off:
			self.last_write_off = self.start_date_time
		super(ServiceLogisticsUparam, self).save()
	def get_cost(self):
		if self.fixed_payment:
			return self.fixed_payment
		else:
			cost = decimal.Decimal(0)
			for obj in ServiceObject.objects.filter(contract=self):
				cost += obj.cost
			cost *= self.coefficient
			return cost
	get_cost.short_description = u'Стоимость'


class ServiceObject(models.Model):
	contract = models.ForeignKey(ServiceLogisticsUparam, verbose_name=u'Контракт')
	name = models.CharField(u'Имя', max_length=60)
	cost = models.DecimalField(u'Стоимость', max_digits=8, decimal_places=2, default=0.0)
	description = models.CharField(u'Описание', max_length=255)
	def __unicode__(self):
		return self.name

class Task(models.Model):
	id = models.AutoField(primary_key=True)
	type = models.IntegerField(u'Тип', choices=TASK_TYPE, default=0)
	type_place = models.IntegerField(u'Место решения', choices=TASK_TYPE_PLACE, default=0)
	description = models.TextField(u'Описание')
	contract = models.ForeignKey(ServiceLogisticsUparam, verbose_name=u'Контракт')
	engineer = models.ForeignKey(Engineer, verbose_name=u'Инженер', null=True, blank=True, help_text=u'Если это поле не заполнено система сама назначит инженера')
	status = models.IntegerField(choices=TASK_STATUS, verbose_name=u'Статус', default=0)
	confirmed = models.NullBooleanField(u'Подтверждена инженером')
	def __unicode__(self):
		return u"Задание %s [%s]" % (self.id, self.contract)
	class Meta:
		verbose_name = u'задание'
		verbose_name_plural = u'Задания'
	def save(self, admin=None):
		th = TaskHistory(
				date_time = datetime.now(),
				admin = admin,

				status = self.status,
				confirmed = self.confirmed,
			)
		# Payment to engineer and dispatcher whan task is success
		if int(self.status) == 3 and self.id and self.engineer:
			old_task = Task.objects.get(pk=self.id)
			# Only if it is first set task status to success
			if TaskHistory.objects.filter(task=old_task, status=3).count() == 0:
				cost = self.get_cost()
				engineer = self.engineer

				# get dispatcher
				user_who_create = TaskHistory.objects.filter(task=old_task).order_by('date_time')[0].admin
				if Dispatcher.objects.filter(user=user_who_create).count() == 1:
					dispatcher = Dispatcher.objects.get(user=user_who_create)
				else:
					dispatcher = None

				if self.type_place == 0: # Solved in office
					e_percent = engineer.office_percent_payment
					e_base_payment = engineer.office_base_payment
					if dispatcher:
						d_percent = dispatcher.office_percent_payment
						d_base_payment = dispatcher.office_base_payment
				elif self.type_place == 1: # Solved with going to client
					e_percent = engineer.remote_percent_payment
					e_base_payment = engineer.remote_base_payment
					if dispatcher:
						d_percent = dispatcher.remote_percent_payment
						d_base_payment = dispatcher.remote_base_payment

				engineer.balance += (cost * e_percent / decimal.Decimal(100) + e_base_payment)
				engineer.save()

				if dispatcher:
					dispatcher.balance += (cost * d_percent / decimal.Decimal(100) + d_base_payment)
					dispatcher.save()
				client = self.contract.client
				client.balance -= cost
				client.save()

		super(Task, self).save()
		th.task = self
		th.save()
		if not self.engineer:
			self.assign_engineers_to_new_tasks()

	def assign_engineers_to_new_tasks(self):
		"""Assign engineers for tasks without engineer"""
		for task in Task.objects.filter(engineer=None):
			engineers = task.contract.client.engineer_set.filter(status__gte=1).order_by('status')
			if engineers:
				task.engineer = engineers[0]
				task.status = 0
				task.save()

	def assign_engineers_to_rejected_tasks(self):
		"""Reassign engineers for rejected tasks"""
		for task in Task.objects.filter(confirmed=False):
			engineers =  task.contract.client.engineer_set.all()
			if engineers:
				continue
			try:
				old_engineer_index = engineers.index(task.engineer)
				if len(engineers) <= old_engineer_index + 1:
					new_engineer_index = 0
				else:
					new_engineer_index = old_engineer_index + 1
			except:
				new_engineer_index = 0
			task.engineer =	engineers[new_engineer_index]
			task.confirmed = None
			task.status = 0
			task.save()
	def get_cost(self):
		contract = self.contract
		if contract.type == 0: # One-time contract
			return contract.get_cost()
		elif contract.type == 1: # Abonement
			if contract.unlimit_service:
				return None
			else:
				return contract.get_cost()/(contract.planned_service_count+contract.exrta_service_count)
	get_cost.short_description = u'Стоимость'


### History models
from datetime import datetime
class TaskHistory(models.Model):
	id = models.AutoField(primary_key=True)
	date_time = models.DateTimeField(u'Дата и время', default=datetime.now())
	admin = models.ForeignKey(User, verbose_name=u'Админ', null=True, blank=True, related_name='logistics_task_history')

	task = models.ForeignKey(Task, verbose_name=u'Задание')
	status = models.IntegerField(choices=TASK_STATUS, verbose_name=u'Статус')
	confirmed = models.NullBooleanField(u'Подтверждена инженером')

	class Meta:
		verbose_name = u'история заданий'
		verbose_name_plural = u'История заданий'

class EngineerHistory(models.Model):
	id = models.AutoField(primary_key=True)
	date_time = models.DateTimeField(default=datetime.now())
	admin = models.ForeignKey(User, verbose_name=u'Админ', null=True, blank=True)

	engineer = models.ForeignKey(Engineer, verbose_name=u'Инженер')

	balance = models.DecimalField(u'Баланс', max_digits=10, decimal_places=2, default=0.0)
	status = models.IntegerField(u'Статус', choices=ENGINEER_STATUS, default=0)
	location = models.CharField(u'Местоположение', max_length=200, null=True, blank=True)
	target = models.ForeignKey(Client, verbose_name=u'Цель', null=True, blank=True, related_name='target_set2')

	class Meta:
		verbose_name = u'история инженеров'
		verbose_name_plural = u'История инженеров'

class DispatcherHistory(models.Model):
	id = models.AutoField(primary_key=True)
	date_time = models.DateTimeField(default=datetime.now())
	admin = models.ForeignKey(User, verbose_name=u'Админ', null=True, blank=True)

	dispatcher = models.ForeignKey(Dispatcher, verbose_name=u'Диспетчер')

	balance = models.DecimalField(u'Баланс', max_digits=10, decimal_places=2, default=0.0)
	status = models.IntegerField(u'Статус', choices=ENGINEER_STATUS, default=0)

	class Meta:
		verbose_name = u'история диспетчеров'
		verbose_name_plural = u'История диспетчеров'
### END History models

class ServiceLogisticsStat(service.ServiceStat):
	task = models.ForeignKey(Task, verbose_name=u'Задание')
	engineer = models.ForeignKey(Engineer, verbose_name=u'Инженер')
	dispatcher = models.ForeignKey(Dispatcher, verbose_name=u'Диспетчер')
	client_payment = models.DecimalField(u'Платеж клиента', max_digits=10, decimal_places=2, default=0.0)
	engineer_base_payment = models.DecimalField(u'Базовый платеж инженеру', max_digits=10, decimal_places=2, default=0.0)
	engineer_percent_payment = models.DecimalField(u'Процент инженеру', max_digits=10, decimal_places=2, default=0.0)
	dispatcher_base_payment = models.DecimalField(u'Базовый платеж диспетчеру', max_digits=10, decimal_places=2, default=0.0)
	dispatcher_percent_payment = models.DecimalField(u'Процент диспетчеру', max_digits=10, decimal_places=2, default=0.0)
	class Meta:
		db_table = u'service_logistics_stats'
		verbose_name = u'статистика платежей'
		verbose_name_plural = u'Статистика платежей'


